# Copyright 2018-2013, Agriculture and Biology
# Author: Sai Wang
#
# This code is part of the aimap package, and is governed by its licence.
# Please see the LICENSE file that should have been included as part of
# this package.

"""Configuration settings for the aimap package.
"""


# Defaults assume that common binaries are on the $PATH
TRIM_GALORE_DEFAULT = "trim_galore"
BWA_DEFAULT = "bwa"
SAMTOOLS_DEFAULT = "samtools"
BCFTOOLS_DEFAULT = "bcftools"
